@@ -37,7 +37,10 @@ DEFAULT_VERIFY_DESC = 'Passcrow Data'
 DEFAULT_EXP_DAYS = 365  # Default requested escrow expiration
 DEFAULT_TMO_MINS = 30   # Default requested verification timeout
 
-VERIFICATION_PREFIXES = 'ABCDEFGHJKLMNPQRSTUVWXYZ23456789'
+# Note: All caps! And oOlL must never be on the list!
+#       We put digits first so codes will usually be entirely numeric;
+#       for usability and similarity to popular systems.
+VERIFICATION_PREFIXES = '1234567890ABCDEFGHJKMNPQRSTUVWXYZ'
 
 EPHEMERAL_ONLY = 1
 EPHEMERAL_BOTH = 2
@@ -159,12 +162,15 @@ class PasscrowServerPolicy:
         <KIND_LIST> via <SERVER_NAME>
 
     Where the KIND_LIST is a comma separated list of protocols such
-    as `mailto`, `tel` or `sms`.
+    as `email`, `tel` or `sms`.
     """
     def __init__(self):
         self.kinds = []
         self.server = None
 
+    def __repr__(self):
+        return '<%s=%s>' % (type(self).__name__, self)
+
     def __str__(self):
         return '%s via %s' % (', '.join(self.kinds), self.server)
 
@@ -178,14 +184,14 @@ class PasscrowIdentityPolicy:
     """
     Verification policies are expressed as text like so:
 
-        <ID>[, warn=<ID>][, notify=<ID>][ via <HELPER_NAME>]
+        <ID>[, notify=<ID>][ via <HELPER_NAME>]
 
     Example:
 
-        mailto:a@a.org via passcrow.example.org
-        mailto:a@a.org, warn=-, notify=b@b.com via passcrow.org
+        email:a@a.org via passcrow.example.org
+        email:a@a.org, notify=b@b.com via passcrow.org
 
-    An <ID> is contact URL, such as mailto:bre@example.org or
+    An <ID> is contact URL, such as email:bre@example.org or
     tel:+3545885522. For convenience, white-space is collapsed and
     URL prefixes are added automatically to bare e-mail addresses
     and phone numbers. The commas, and order of elements, and the
@@ -196,7 +202,6 @@ class PasscrowIdentityPolicy:
     """
     def __init__(self):
         self.id = None
-        self.warn = None
         self.notify = None
         self.server = None
         self.timeout = None
@@ -204,10 +209,11 @@ class PasscrowIdentityPolicy:
 
     usable = property(lambda s: s.server and s.id)
 
+    def __repr__(self):
+        return '<%s=%s>' % (type(self).__name__, self)
+
     def __str__(self):
         txt = self.id
-        if self.warn:
-            txt += ', warn=' + self.warn
         if self.notify:
             txt += ', notify=' + self.notify
         if self.server:
@@ -230,13 +236,16 @@ class PasscrowIdentityPolicy:
         kinds = set([self.id.kind])
         if self.notify:
             kinds.add(self.notify.kind)
-        if self.warn:
-            kinds.add(self.warn.kind)
+        used_psp = None
         for psp in defaults.servers:
             psp_kinds = set(psp.kinds)
             if not (kinds - psp_kinds):
                 pip.server = psp.server
+                used_psp = psp
                 break
+        if defaults.servers and used_psp:
+            defaults.servers.remove(used_psp)
+            defaults.servers.append(used_psp)
         return pip
 
     def get_timeout(self):
@@ -250,8 +259,6 @@ class PasscrowIdentityPolicy:
             self.id, self.server = self.id.rsplit(' via ', 1)
         if ', notify=' in self.id:
             self.id, self.notify = self.id.rsplit(', notify=')
-        if ', warn=' in self.id:
-            self.id, self.warn = self.id.rsplit(', warn=')
 
         dpip = self._default_pip(defaults)
 
@@ -260,36 +267,61 @@ class PasscrowIdentityPolicy:
         elif dpip and not self.notify:
             self.notify = dpip.notify
 
-        if self.warn == '-':
-            self.warn = None
-        elif dpip and not self.warn:
-            self.warn = dpip.warn
-
         self.id = Identity(self.id)
-        self.warn = Identity(self.warn) if self.warn else None
         self.notify = Identity(self.notify) if self.notify else None
 
-        if dpip and not self.server:
-            dpip = self._choose_server(dpip, defaults)
+        if not self.server:
+            if not dpip:
+                dpip = PasscrowIdentityPolicy()
+            if defaults:
+                dpip = self._choose_server(dpip, defaults)
             self.server = dpip.server
 
         return self
 
 
-class PasscrowClientPolicy:
-    def __init__(self, idps=None,
+class PasscrowRecoveryPolicy:
+    def __init__(self,
+            idps=None,
             n=None, m=None,
-            servers=None,
             expiration_days=None,
-            timeout_minutes=None):
-        self.idps = idps or []
-        # Set our default ratio
+            timeout_minutes=None,
+            servers=None,
+            defaults=None,
+            default_policy=False):
+        # Set general policy preferences
         self.n = DEFAULT_N if (n is None) else n
         self.m = DEFAULT_M if (m is None) else m
+        self.expiration_days = expiration_days or DEFAULT_EXP_DAYS
+        self.timeout_minutes = timeout_minutes or DEFAULT_TMO_MINS
+
+        if not default_policy:
+            if defaults:
+                if not expiration_days:
+                    self.expiration_days = defaults.expiration_days
+                if not timeout_minutes:
+                    self.timeout_minutes = defaults.timeout_minutes
+            elif servers:
+                # Create a default policy from the provided server list
+                defaults = PasscrowRecoveryPolicy(
+                    servers=servers,
+                    default_policy=True)
+
+        # Set identity list, parsing if necessary
+        self.idps = []
+        if idps:
+            self.idps = [
+                (i if isinstance(i, PasscrowIdentityPolicy)
+                    else PasscrowIdentityPolicy().parse(i, defaults=defaults))
+                for i in idps]
+
         # This is for default policies only
-        self.servers = servers or []
-        self.expiration_days = expiration_days
-        self.timeout_minutes = timeout_minutes
+        self.servers = []
+        if default_policy and servers:
+            self.servers = [
+                (s if isinstance(s, PasscrowServerPolicy)
+                    else PasscrowServerPolicy().parse(s))
+                for s in servers]
 
     def absolute_ratio(self, reserve=0):
         available = len(self.idps) - reserve
@@ -305,13 +337,16 @@ class PasscrowClientPolicy:
 
         return rn, rm
 
+    def __repr__(self):
+        return '<%s=%s>' % (type(self).__name__, self)
+
     def __str__(self):
         text = '%d/%d of %s' % (
             self.n,
             self.m,
-            ', '.join(str(i) for i in self.idps))
+            (', '.join(str(i) for i in self.idps)) if self.idps else '(ids)')
         if self.servers:
-            text += ' (%s)' % ', '.join(str(sp) for sp in self.servers)
+            text += ' servers=(%s)' % ', '.join(str(sp) for sp in self.servers)
         return text
 
 
@@ -326,28 +361,38 @@ class PasscrowClient:
 
     def __init__(self,
             config_dir=None, data_dir=None, env_override=True,
+            create_dirs=False,
+            load_defaults=True,
+
             default_ids=None,
+            default_servers=None,
             default_n=None,
             default_m=None,
             default_expiration_days=None,
             default_timeout_minutes=None,
-            create_dirs=False, logger=None,
-            sleep_min=None, sleep_max=None,
-            sleep_func=None,
+
+            sleep_min=None, sleep_max=None, sleep_func=None,
+            logging_func=None,
             urlopen_func=None):
-        self.log = logger or print
+
         self.config_dir = config_dir or SHARED_CONFIG_DIR
         self.data_dir = data_dir or SHARED_DATA_DIR
+
         self.sleep_min = DEFAULT_SLEEP_MIN if sleep_min is None else sleep_min
         self.sleep_max = DEFAULT_SLEEP_MAX if sleep_max is None else sleep_max
-        self.expiration_days = default_expiration_days or DEFAULT_EXP_DAYS
-        self.timeout_minutes = default_timeout_minutes or DEFAULT_TMO_MINS
-        self.default_policy = PasscrowClientPolicy(
-             idps=default_ids or [], n=default_n, m=default_m)
-
+        self.log = logging_func or print
         self.sleep = sleep_func or time.sleep
         self.urlopen = urlopen_func or _default_urlopen
 
+        self.default_policy = PasscrowRecoveryPolicy(
+            idps=default_ids,
+            servers=default_servers,
+            n=default_n,
+            m=default_m,
+            expiration_days=default_expiration_days,
+            timeout_minutes=default_timeout_minutes,
+            default_policy=True)
+
         # FIXME: Cache these on disk, load on startup
         self.server_policies = {}
 
@@ -357,16 +402,21 @@ class PasscrowClient:
             self.config_dir = os.getenv('PASSCROW_CONFIG', self.config_dir)
             self.data_dir = os.getenv('PASSCROW_DATA', self.data_dir)
 
+        just_created = False
         for path in (self.config_dir, self.data_dir):
             if not os.path.exists(path):
                 if not create_dirs:
                     raise OSError('Directory not found: %s' % path)
                 pmkdir(path, 0o700)
                 self.log('%s: Created %s' % (self, path))
+                just_created = True
         try:
-            self.load_default_policy()
+            if load_defaults:
+                self.load_default_policy()
         except (OSError, IOError):
             pass
+        if just_created:
+            self.save_default_policy()
 
     def __str__(self):
         if self.config_dir != self.data_dir:
@@ -435,12 +485,10 @@ class PasscrowClient:
 
         erp = EscrowRequestParameters()
         erp.kind = idp.id.split(':')[0]
-        erp.expiration = (policy.expiration_days or self.expiration_days)
+        erp.expiration = policy.expiration_days
         erp.expiration *= (24 * 3600)
         erp.payment = self._make_payment(idp, erp.expiration, str(erd))
         erp.expiration += int(time.time())
-        if idp.warn:
-            erp.warnings_to = idp.warn
         if escrow_id:
             erp.prefer_id = escrow_id
         erp.encrypt(random_aesgcm_key())
@@ -558,6 +606,10 @@ class PasscrowClient:
         failures = []
         def prep(task, delay):
             idp, share, mer_kwargs = task
+            if not idp.id:
+                raise ValueError('Invalid ID: %s' % idp.id)
+            if not idp.server:
+                raise ValueError('Need a server for id %s' % idp.id)
             self.log(
                 'Slept %3.3ds. Escrow share %s for %s with %s'
                 % (delay, share.split('-')[0], idp.id, idp.server))
@@ -635,11 +687,17 @@ class PasscrowClient:
                 failures.append(e)
         return (not failures)
 
+    def wanted_language(self):
+        # FIXME: How do we know which language the user wants?
+        #        This approach is useful for testing, but not much else.
+        return os.getenv('PASSCROW_LANGUAGE', 'en')
+
     def verify(self, pack, quick=False, now=None):
         ok, failures = True, []
         responses = {}
         def prep(prefix_esc, delay):
             vreq = VerificationRequest()
+            vreq.language = self.wanted_language()
             vreq.prefix, esc = prefix_esc
             vreq.escrow_data_id = _id = esc.response.escrow_data_id
             vreq.escrow_data_key = esc.recovery_key
@@ -684,6 +742,10 @@ class PasscrowClient:
         def fmt_fail(prefix_esc, server, vreq, _id, e):
             return '%s on %s: %s' % (_id, server, e)
 
+        # Transform in case users are bad at typing
+        codes = [code.upper().replace('O', '0').replace('L', '1')
+                 for code in codes]
+
         codes = dict((code[:1].upper(), code) for code in codes)
         tasks = [(codes[c], esc)
             for c, esc in pack.prefixed_escrow_list()
@@ -738,13 +800,11 @@ class PasscrowClient:
                     timeout_minutes = int(what)
                 else:
                     raise ValueError('Unknown setting: %s' % op)
-        self.default_policy = PasscrowClientPolicy(
-            idps=idps, n=n, m=m, servers=servers)
+        self.default_policy = PasscrowRecoveryPolicy(
+            idps=idps, n=n, m=m, servers=servers, default_policy=True)
         if expiration_days:
-            self.expiration_days = expiration_days
             self.default_policy.expiration_days = expiration_days
         if timeout_minutes:
-            self.timeout_minutes = timeout_minutes
             self.default_policy.timeout_minutes = timeout_minutes
 
     def save_default_policy(self):
@@ -793,7 +853,7 @@ class PasscrowClient:
 # and deletes all data after 30 days, no matter what expiration has been
 # requested (or promised).
 #
-#server: mailto via passcrow-test.mailpile.is
+#server: email via passcrow-test.mailpile.is
 #server: sms, tel via sms.passcrow.example.org
 #
 #expiration_days: 365   # How long we request key fragments persist in escrow
