@@ -10,6 +10,11 @@ def route_passcrow_api(app, server):
             info['user'] = request.remote_user
         return ', '.join('%s=%s' % (k, v) for k, v in info.items())
 
+    def passcrow_stats():
+        return Response(
+            str(server.handle(user_info(), 'stats', request.data or '{}')),
+            content_type="application/json")
+
     def passcrow_policy():
         return Response(
             str(server.handle(user_info(), 'policy', request.data or '{}')),
@@ -20,5 +25,6 @@ def route_passcrow_api(app, server):
             str(server.handle(user_info(), rpc_method, request.data)),
             content_type="application/json")
 
+    app.route('/passcrow/stats', methods=['GET', 'POST'])(passcrow_stats)
     app.route('/passcrow/policy', methods=['GET', 'POST'])(passcrow_policy)
     app.route('/passcrow/<rpc_method>', methods=['POST'])(passcrow_api)
